  // crlf2lf.lex
  // flex input to convert CRLF to LF, in the context of sftpc
  // copyright Scott McPeak, 1999  Terms of use are as specified in license.txt.

  // ------- Flex options ------------
%option noyywrap
%option nounput
%option outfile="crlf2lf.cpp"
%option prefix="crlf2lf_"

  // --------- C++ declarations ------------
  #include "crlflex.h"     // common declarations with crlf2lf.lex

  // say what the scanner's interface is
  #define YY_DECL void inner_CRLF_to_LF(StreamOutputDest &dest,    \
                                        int &bareCRs, int &bareLFs)
  #define yyterminate() return


  // ---------- rules ------------
%%

"\r\n" {
  dest.write("\n", 1);
}

"\r" {
  // a bare \r isn't supposed to appear in CRLF
  // text.. I choose to eat it, but not silently
  bareCRs++;
}

"\n" {
  // a bare \n isn't supposed to appear either
  bareLFs++;
}

[^\r\n]+ {
  dest.write(yytext, yyleng);
}


%%
// ---------------- additional C++ code ---------------

#include "nonport.h"    // LOCKER

void CRLF_to_LF(StreamInputSource &source,
                StreamOutputDest &dest,
                int &bareCRs, int &bareLFs)
{
  LOCKER
  streamSource = &source;
  inner_CRLF_to_LF(dest, bareCRs, bareLFs);
  streamSource = NULL;
}


#ifdef TEST_CRLF2LF

#include "filesrc.h"      // FileInputSource, FileOutputDest
#include "test.h"         // USUAL_MAIN

int entry(int argc, char *argv[])
{
  if (argc <= 2) {
    printf("usage: crlf2lf input-crlf-file output-lf-file\n");
    return 0;
  }

  FILE *in = fopen(argv[1], "rb");
  FILE *out = fopen(argv[2], "wb");
  if (!in || !out) {
    printf("error opening one of the input files\n");
    return 2;
  }

  FileInputSource src(in);
  FileOutputDest dest(out);
  int bareCRs=0, bareLFs=0;

  CRLF_to_LF(src, dest, bareCRs, bareLFs);

  printf("bareCRs=%d, bareLFs=%d\n", bareCRs, bareLFs);

  fclose(in);
  fclose(out);

  return 0;
}

ARGS_MAIN

#endif // TEST_CRLF2LF

